/*
 *           Grain - a minimal windowing system, version 1.1
 *
 *                            Written by
 *                          Harald Winroth
 *      Computational Vision and Active Perception Lab (CVAP)
 *                  Royal Institute of Technology
 *                            Stockholm
 * 
 *                Copyright (c) 1998 Harald Winroth
 *
 * See the file "Copying" for information on usage and redistribution of 
 * this file, and for a disclaimer of all warranties. Please send bug 
 * reports and direct all inquiries to harald@nada.kth.se.
 *
 */

#ifndef GRAIN_GEOMETRY_HH
#define GRAIN_GEOMETRY_HH

#include <iostream.h>

struct GrainCoord
{
    GrainCoord();
    GrainCoord(long x, long y);
    bool operator==(const GrainCoord& coord) const;
    long x, y;
};

GrainCoord operator+(const GrainCoord& c1, const GrainCoord& c2);
GrainCoord operator-(const GrainCoord& c1, const GrainCoord& c2);
GrainCoord& operator+=(GrainCoord& c1, const GrainCoord& c2);
GrainCoord& operator-=(GrainCoord& c1, const GrainCoord& c2);
ostream& operator<<(ostream& os, const GrainCoord& c);

struct GrainRect
{
    GrainRect();
    GrainRect(const GrainCoord& position, const GrainCoord& size);
    static GrainRect with_corners(const GrainCoord& corner1, 
				   const GrainCoord& corner2);
    bool operator==(const GrainRect& rect) const;
    GrainCoord position, size;
};

GrainRect intersection(const GrainRect& r1, const GrainRect& r2);
bool intersects(const GrainRect& r1, const GrainRect& r2);
bool inside(const GrainCoord& p, const GrainRect& r);
GrainRect hull(const GrainRect& r1, const GrainRect& r2);
GrainRect expand(const GrainRect& r, int steps);
ostream& operator<<(ostream& os, const GrainRect& r);

#endif // GRAIN_GEOMETRY_HH
