/*
 *           Grain - a minimal windowing system, version 1.1
 *
 *                            Written by
 *                          Harald Winroth
 *      Computational Vision and Active Perception Lab (CVAP)
 *                  Royal Institute of Technology
 *                            Stockholm
 * 
 *                Copyright (c) 1998 Harald Winroth
 *
 * See the file "Copying" for information on usage and redistribution of 
 * this file, and for a disclaimer of all warranties. Please send bug 
 * reports and direct all inquiries to harald@nada.kth.se.
 *
 */

#include <grain/window.hh>

GrainColor::GrainColor() : 
    r(0), g(0), b(0), fill(false)
{}

GrainColor::GrainColor(unsigned char red, unsigned char green, 
		       unsigned char blue) :
    r(red), g(green), b(blue), fill(true)
{}

bool GrainColor::operator==(const GrainColor& color) const
{
    return r == color.r && g == color.g && b == color.b && fill == color.fill;
}

const GrainColor GrainColor::transparent;
const GrainColor GrainColor::black(0, 0, 0);
const GrainColor GrainColor::white(255, 255, 255);
const GrainColor GrainColor::red(255, 0, 0);
const GrainColor GrainColor::green(0, 255, 0);
const GrainColor GrainColor::blue(0, 0, 255);

GrainWindow::GrainWindow() : 
    event_handler_(0) 
{}

void GrainWindow::set_event_handler(GrainWindowEventHandler* event_handler)
{
    event_handler_=event_handler;
}

GrainWindowEventHandler* GrainWindow::event_handler() const
{
    return event_handler_;
}

void GrainWindow::refresh()
{
    refresh(GrainRect(GrainCoord(0, 0), bounds().size));
}

GrainWindowingSystem::Exception::Exception(const string& what) :
    what_(what)
{}

const char* GrainWindowingSystem::Exception::what () const
{ 
    return what_.c_str();
}

void GrainWindowingSystem::error(const string& what)
{
    throw Exception(what);
}

