#include <grain/grain.hh>

string text("Hello World");
string font("9x15");

class EventHandler : public GrainWindowEventHandler
{
public:
    EventHandler();
    void press_event(GrainWindow* w, const GrainCoord& pos, size_t button, 
		     const set<Modifier>& modifiers);
    void release_event(GrainWindow* w, const GrainCoord& pos, size_t button);
    void refresh_event(GrainWindow* w, const GrainRect& bounds);
    void key_event(GrainWindow* w, const string& value, 
		   const set<Modifier>& modifiers);
private:
    bool inverted_;
};

int main()
{
    try {
	GrainXWindowingSystem system;
	GrainWindow* window=system.create_base_window(
	    GrainRect(GrainCoord(0, 0), system.text_size(font, text)), 
	    false, "Test");
	EventHandler handler;
	window->set_event_handler(&handler);
	system.enter_event_loop();
    }
    catch(const exception& e) {
	cerr << "main: " << e.what() << endl;
	return 1;
    }

    return 0;
}

EventHandler::EventHandler() : inverted_(false)
{}

void EventHandler::press_event(GrainWindow* w, const GrainCoord& pos, 
			       size_t button, const set<Modifier>& modifiers)
{
    inverted_=true;
    w->refresh(w->bounds()); // generate a refresh event
}
 
void EventHandler::release_event(GrainWindow* w, const GrainCoord& pos, 
				 size_t button)
{
    inverted_=false;
    w->refresh(w->bounds()); // generate a refresh event
}
 
void EventHandler::refresh_event(GrainWindow* w, const GrainRect& bounds)
{
    // We ignore 'bounds' here and always redraw the entire window.
    GrainColor background=GrainColor::white;
    GrainColor foreground=GrainColor::black;
    if (inverted_)
	swap(background, foreground);
    w->draw_rectangle(w->bounds(), background, background);
    w->draw_text(GrainCoord(0, 0), text, font, foreground);
}

void EventHandler::key_event(GrainWindow* w, const string& value, 
			     const set<Modifier>& modifiers)
{
    if (value == "q")
	w->system().exit_event_loop();
}
    
