#include <grain/grain.hh>

class EventHandler : public GrainWindowEventHandler
{
public:
    EventHandler() : popup_(0) {}

    void press_event(GrainWindow* w, const GrainCoord& pos, size_t button, 
		     const set<Modifier>& modifiers)
    {
	cerr << "press_event: w=" << w << ", pos=" << pos << 
	    ", button=" << button << ", modifiers=";
	copy(modifiers.begin(), modifiers.end(), 
	     ostream_iterator<Modifier>(cerr));
	cerr << endl;
#if 1
	GrainRect b(w->absolute(pos), GrainCoord(50, 100));
	cerr << "w->bounds()=" << w->bounds() << endl;
	cerr << "b=" << b << endl;
	GrainWindowingSystem& sys=w->system();
	if (popup_)
	    delete popup_;
	popup_=sys.create_pop_up_window(b);
	popup_->set_event_handler(this);
	//w->ungrab_pointer();
	cerr << "  created pop-up window " << popup_ << endl;
#endif
    }
    void release_event(GrainWindow* w, const GrainCoord& pos, size_t button)
    {
	cerr << "release_event: w=" << w << ", pos=" << pos << 
	    ", button=" << button << endl;
#if 1
	cerr << "  deleting created pop-up window " << popup_ << endl;
	delete popup_;
	popup_=0;
#endif
    }
    void motion_event(GrainWindow* w, const GrainCoord& pos)
    {
	cerr << "motion_event: w=" << w << ", pos=" << pos << endl;
    }
    void enter_event(GrainWindow* w, const GrainCoord& pos)
    {
	cerr << "enter_event: w=" << w << ", pos=" << pos << endl;
    }
    void leave_event(GrainWindow* w, const GrainCoord& pos)
    {
	cerr << "leave_event: w=" << w << ", pos=" << pos << endl;
    }
    void key_event(GrainWindow* w, const string& value, 
		   const set<Modifier>& modifiers)
    {
	cerr << "key_event: w=" << w << ", value=" << value << ", modifiers=";
	copy(modifiers.begin(), modifiers.end(), 
	     ostream_iterator<Modifier>(cerr));
	cerr << endl;
	if (value == "q")
	    w->system().exit_event_loop();
	else if (value == "c") {
	    popup_=w->system().create_pop_up_window(
		GrainRect(GrainCoord(0, 0), GrainCoord(50, 100)));
	    popup_->set_event_handler(w->event_handler());
	    cerr << "  created pop-up window " << popup_ << endl;
	}
	else if (value == "d") {
	    cerr << "  deleting created pop-up window " << popup_ << endl;
	    delete popup_;
	    popup_=0;
	}
	else if (value == "g") {
	    cerr << "  grabbing pointer" << endl;
	    w->grab_pointer();
	}
	else if (value == "r") {
	    cerr << "  releasing pointer" << endl;
	    w->ungrab_pointer();
	}
    }
    void refresh_event(GrainWindow* w, const GrainRect& bounds)
    {
	cerr << "refresh_event: w=" << w << ", bounds=" << bounds << endl;
#if 1
	w->draw_rectangle(GrainRect(GrainCoord(10, 10), GrainCoord(50, 25)), 
			  GrainColor::blue, GrainColor::green);
	w->draw_point(GrainCoord(9, 9), GrainColor::red);
	w->draw_point(GrainCoord(60, 35), GrainColor::red);
	w->draw_point(GrainCoord(0, 0), GrainColor::blue);
	w->draw_point(w->bounds().size-GrainCoord(1, 1), GrainColor::blue);

	string text("Hello world");
	string font_name("9x15");
	w->draw_rectangle(
	    GrainRect(GrainCoord(10, 80), 
		      w->system().text_size(font_name, text)),
	    GrainColor::blue, GrainColor::green);
	w->draw_text(GrainCoord(10, 80), text, font_name, GrainColor::red);
	w->draw_point(GrainCoord(10, 80), GrainColor::green);
#elif 0
	w->draw_line(GrainCoord(0, 9), GrainCoord(9, 9), 
			  GrainColor::blue);
	w->draw_line(GrainCoord(0, 10), GrainCoord(9, 10), GrainColor::blue);
	w->draw_point(GrainCoord(0, 10), GrainColor::red);
	w->draw_point(GrainCoord(9, 10), GrainColor::red);
#else
	GrainRect b=w->bounds();
	long x=0, y=0;
	// Horizontal lines
	while (y<b.size.y) {
	    w->draw_line(GrainCoord(0, y), GrainCoord(b.size.x-1, y), 
			 GrainColor::blue);
	    y+=10;
	}
	// Vertical lines
	while (x<b.size.x) {
	    w->draw_line(GrainCoord(x, 0), GrainCoord(x, b.size.y-1), 
			 GrainColor::red);
	    x+=10;
	}
#endif
    }

private:
    GrainWindow* popup_;
};

int main(int argc, char* argv[])
{
    if (argc < 2) {
	cerr << "usage: " << argv[0] << " display" << endl;
	exit(1);
    } 
    try {
	GrainXWindowingSystem sys(argv[1]);
	GrainWindow* base=sys.create_base_window(
	    GrainRect(GrainCoord(0, 0), GrainCoord(300, 150)), true, 
	    "Simple Drawing");
	EventHandler eh;
	base->set_event_handler(&eh);
	GrainWindow* sub=base->create_subwindow(
	    GrainRect(GrainCoord(50, 75), GrainCoord(300, 150)));
	sub->set_event_handler(&eh);
	sys.enter_event_loop();
    }
    catch(const exception& e) {
	cerr << "main: " << e.what() << endl;
	return 1;
    }
    return 0;
}
